@@ -612,6 +612,32 @@ class AutosubmitConfig(object):
                 data_fixed["JOBS"][job]["ADDITIONAL_FILES"].append(file)
 
         return data_fixed
+
+    def dict_replace_value(self,d: dict, old: str, new: str) -> dict:
+        x = {}
+        for k, v in d.items():
+            if isinstance(v, dict):
+                v = self.dict_replace_value(v, old, new)
+            elif isinstance(v, list):
+                v = self.list_replace_value(v, old, new)
+            elif isinstance(v, str):
+                v = v.replace(old, new)
+            x[k] = v
+        return x
+
+    def list_replace_value(self,l: list, old: str, new: str) -> list:
+        x = []
+        for e in l:
+            if isinstance(e, list):
+                e = self.list_replace_value(e, old, new)
+            elif isinstance(e, dict):
+                e = self.dict_replace_value(e, old, new)
+            elif isinstance(e, str):
+                e = e.replace(old, new)
+            x.append(e)
+        return x
+
+
     def unify_conf(self):
         '''
         Unifies all configuration files into a single dictionary. Custom files will be able to override the default configuration.
@@ -654,6 +680,7 @@ class AutosubmitConfig(object):
             self.deep_read_loops(self.experiment_data)
             self.parse_data_loops(self.experiment_data, self.data_loops)
         self.dynamic_variables = list(set(self.dynamic_variables))
+        self.experiment_data = self.sustitute_dynamic_variables(self.experiment_data,max_deep=25)
 
     def parse_data_loops(self,exp_data,data_loops):
         section_data = list()
@@ -723,6 +750,75 @@ class AutosubmitConfig(object):
                 placeholders.append(posible_placeholder.strip("%"))
         return placeholders
 
+    def check_dict_keys_type(self,parameters):
+        '''
+        Check if keys are plain into 1 dimension, checks for 33% of dict to ensure it.
+        :param parameters: experiment parameters
+        :return:
+        '''
+        amount_of_keys_to_check = int(len(parameters)/3)+1
+        count_dot = 0
+        count = 0
+        for key in parameters.keys():
+            if "." in key:
+                count_dot+=1
+            count+=1
+            if count >= amount_of_keys_to_check:
+                break
+        if count_dot >= int(count/2)+1:
+            dict_keys_type = "long"
+        else:
+            dict_keys_type = "short"
+        return dict_keys_type
+    def sustitute_dynamic_variables(self,parameters=None,max_deep=25,dict_keys_type=None):
+        """
+        Substitute dynamic variables in the experiment data
+        :parameter
+        :return:
+        """
+
+        if parameters is None:
+            parameters = self.deep_parameters_export(self.experiment_data)
+        # Check if the parameters key provided are long(%DEFAULT.EXPID%) or short(DEFAULT[EXPID]) if it is not specified.
+        if dict_keys_type is None:
+            dict_keys_type = self.check_dict_keys_type(parameters)
+        backup_variables = self.dynamic_variables
+        while len(self.dynamic_variables) > 0 and max_deep > 0:
+            dynamic_variables = []
+            for dynamic_var in self.dynamic_variables:
+                #get value of placeholder with  name without %%
+                if dict_keys_type == "long":
+                    value = parameters.get(str(dynamic_var[1][1:-1]),None)
+                else:
+                    keys = dynamic_var[1].split(".")
+                    aux_dict = parameters
+                    for k in keys:
+                        k = k.strip("%")
+                        aux_dict = aux_dict.get(k,{})
+                    if len(aux_dict) > 0:
+                        value = aux_dict
+                    else:
+                        value = None
+                if value is not None:
+                    if dict_keys_type == "long":
+                        dict_key = parameters.get(str(dynamic_var[0]), {})
+                        if len(dict_key) > 0:
+                            substituted = True
+                            parameters[str(dynamic_var[0])] = value
+                        else:
+                            substituted = False
+                    else:
+                        # See input and output below todo
+                        substituted = False
+                        parameters = self.dict_replace_value(parameters, dynamic_var[1], value)
+                else:
+                    substituted = False
+                if not substituted:
+                    dynamic_variables.append(dynamic_var)
+            self.dynamic_variables = dynamic_variables
+            max_deep = max_deep - 1
+        self.dynamic_variables = backup_variables
+        return parameters
     def sustitute_placeholder_variables(self,key,val,parameters):
         substituted = False
         data = parameters
@@ -756,30 +852,38 @@ class AutosubmitConfig(object):
             if key == "FOR":
                 self.data_loops.append(for_keys)
             elif isinstance(val, collections.abc.Mapping ):
-                self.deep_read_loops(data.get(key, {}),for_keys+[key],long_key=key+".")
+                self.deep_read_loops(data.get(key, {}),for_keys+[key],long_key=long_key+key+".")
 
 
 
 
 
 
-    def check_mandatory_conf_files(self,refresh=False):
+    def check_mandatory_conf_files(self,refresh=False,no_log=False):
         #self.unify_conf()
-        self.check_expdef_conf(refresh)
-        self.check_platforms_conf()
-        self.check_jobs_conf()
-        self.check_autosubmit_conf(refresh)
+        self.check_expdef_conf(refresh,no_log=no_log)
+        self.check_platforms_conf(no_log=no_log)
+        self.check_jobs_conf(no_log=no_log)
+        self.check_autosubmit_conf(refresh,no_log=no_log)
 
-    def check_conf_files(self, running_time=False,first_load=True,refresh=False):
+    def check_conf_files(self, running_time=False,first_load=True,refresh=False,no_log=False):
         """
         Checks configuration files (autosubmit, experiment jobs and platforms), looking for invalid values, missing
         required options. Print results in log
+        :param running_time: True if the function is called during the execution of the program
+        :type running_time: bool
+        :param first_load: True if the function is called during the first load of the program
+        :type first_load: bool
+        :param refresh: True if the function is called during the refresh of the program
+        :type refresh: bool
+        :param no_log: True if the function is called during describe
+        :type no_log: bool
 
         :return: True if everything is correct, False if it finds any error
         :rtype: bool
         """
-
-        Log.info('\nChecking configuration files...')
+        if not no_log:
+            Log.info('\nChecking configuration files...')
         self.ignore_file_path = running_time
         self.ignore_undefined_platforms = running_time
 
@@ -793,7 +897,7 @@ class AutosubmitConfig(object):
         except BaseException as e:
             raise AutosubmitCritical("Unknown issue while checking the configuration files (check_conf_files)",7040,str(e))
         # Annotates all errors found in the configuration files in dictionaries self.warn_config and self.wrong_config.
-        self.check_mandatory_conf_files(refresh)
+        self.check_mandatory_conf_files(refresh,no_log=no_log)
         try:
             if self.get_project_type():
                 self.check_proj()
@@ -802,22 +906,27 @@ class AutosubmitConfig(object):
         # End of checkers.
         # This Try/Except is in charge of  print all the info gathered by all the checkers and stop the program if any critical error is found.
         try:
-            result = self.show_messages()
-            return result
+            if not no_log:
+                result = self.show_messages()
+                return result
         except AutosubmitCritical as e:
             # In case that there are critical errors in the configuration, Autosubmit won't continue.
             if running_time is True:
                 raise AutosubmitCritical(e.message, e.code, e.trace)
             else:
-                Log.warning(e.message)
+                if not no_log:
+                    Log.warning(e.message)
         except Exception as e:
             raise AutosubmitCritical(
                 "There was an error while showing the config log messages", 7014, str(e))
 
-    def check_autosubmit_conf(self,refresh=False):
+    def check_autosubmit_conf(self,refresh=False,no_log=False):
         """
         Checks experiment's autosubmit configuration file.
-
+        :param refresh: True if the function is called during the refresh of the program
+        :type refresh: bool
+        :param no_log: True if the function is called during describe
+        :type no_log: bool
         :return: True if everything is correct, False if it founds any error
         :rtype: bool
         """
@@ -868,18 +977,18 @@ class AutosubmitConfig(object):
                         self.wrong_config["Autosubmit"] += [['mail',
                                                              "invalid e-mail"]]
         if "Autosubmit" not in self.wrong_config:
-            Log.result('{0} OK'.format(
-                os.path.basename(self._conf_parser_file)))
+            if not no_log:
+                Log.result('{0} OK'.format(
+                    os.path.basename(self._conf_parser_file)))
             return True
         else:
-            Log.warning('{0} Issues'.format(
-                os.path.basename(self._conf_parser_file)))
             return True
         return False
 
-    def check_platforms_conf(self):
+    def check_platforms_conf(self,no_log=False):
         """
-        Checks experiment's queues configuration file.
+        Checks experiment's platforms configuration file.
+
         """
         parser_data = self.experiment_data.get("PLATFORMS",{})
         main_platform_found = False
@@ -910,15 +1019,17 @@ class AutosubmitConfig(object):
         if not main_platform_found:
             self.wrong_config["Expdef"] += [["Default","Main platform is not defined! check if [HPCARCH = {0}] has any typo".format(self.hpcarch)]]
         if "Platform" not in self.wrong_config:
-            Log.result('{0} OK'.format(
-                os.path.basename(self._platforms_parser_file)))
+            if not no_log:
+                Log.result('{0} OK'.format(
+                    os.path.basename(self._platforms_parser_file)))
             return True
         return False
 
-    def check_jobs_conf(self):
+    def check_jobs_conf(self,no_log=False):
         """
         Checks experiment's jobs configuration file.
-
+        :param no_log: if True, it doesn't print any log message
+        :type no_log: bool
         :return: True if everything is correct, False if it founds any error
         :rtype: bool
         """
@@ -976,15 +1087,18 @@ class AutosubmitConfig(object):
                 self.wrong_config["Jobs"] += [[section,
                                                "Mandatory RUNNING parameter is invalid"]]
         if "Jobs" not in self.wrong_config:
-            Log.result('{0} OK'.format(
-                os.path.basename(self._jobs_parser_file)))
+            if not no_log:
+                Log.result('{0} OK'.format(os.path.basename(self._jobs_parser_file)))
             return True
         return False
 
-    def check_expdef_conf(self,refresh=False):
+    def check_expdef_conf(self,refresh=False,no_log=False):
         """
         Checks experiment's experiment configuration file.
-
+        :param refresh: if True, it doesn't check the mandatory parameters
+        :type refresh: bool
+        :param no_log: if True, it doesn't print any log message
+        :type no_log: bool
         :return: True if everything is correct, False if it founds any error
         :rtype: bool
         """
@@ -1060,15 +1174,17 @@ class AutosubmitConfig(object):
             elif project_type == 'none':  # debug propouses
                 self.ignore_file_path = False
         if "Expdef" not in self.wrong_config:
-            Log.result('{0} OK'.format(
-                os.path.basename(self._exp_parser_file)))
+            if not no_log:
+                Log.result('{0} OK'.format(
+                    os.path.basename(self._exp_parser_file)))
             return True
         return False
 
-    def check_proj(self):
+    def check_proj(self,no_log=False):
         """
         Checks project config file
-
+        :no_log if True, it doesn't print any log message
+        :type no_log: bool
         :return: True if everything is correct, False if it founds any error
         :rtype: bool
         """
@@ -1085,36 +1201,48 @@ class AutosubmitConfig(object):
                                            "FILE_PROJECT_CONF parameter is invalid"]]
             return False
 
-    def check_wrapper_conf(self,wrappers=dict()):
-            for wrapper_name,wrapper_values in wrappers.items():
-                jobs_in_wrapper = wrapper_values.get('JOBS_IN_WRAPPER',"")
-                if "&" in jobs_in_wrapper:
-                    jobs_in_wrapper = jobs_in_wrapper.split("&")
-                else:
-                    jobs_in_wrapper = jobs_in_wrapper.split(" ")
-                for section in jobs_in_wrapper:
-                    platform_name = self.jobs_data[section].get('PLATFORM',"").upper()
-                    if platform_name == "":
-                        platform_name = self.get_platform().upper()
-                    if platform_name == "LOCAL":
-                        continue
-                    if not self.is_valid_jobs_in_wrapper(wrapper_values):
+    def check_wrapper_conf(self,wrappers=dict(),no_log=False):
+        """
+        Checks wrapper config file
+
+        :param wrappers:
+        :param no_log:
+        :return:
+        """
+        for wrapper_name,wrapper_values in wrappers.items():
+            #continue if it is a global option (non-dicT)
+            if type(wrapper_values) is not dict:
+                continue
+
+            jobs_in_wrapper = wrapper_values.get('JOBS_IN_WRAPPER',"")
+            if "&" in jobs_in_wrapper:
+                jobs_in_wrapper = jobs_in_wrapper.split("&")
+            else:
+                jobs_in_wrapper = jobs_in_wrapper.split(" ")
+            for section in jobs_in_wrapper:
+                platform_name = self.jobs_data[section].get('PLATFORM',"").upper()
+                if platform_name == "":
+                    platform_name = self.get_platform().upper()
+                if platform_name == "LOCAL":
+                    continue
+                if not self.is_valid_jobs_in_wrapper(wrapper_values):
+                    self.wrong_config["WRAPPERS"] += [[wrapper_name,
+                                                      "JOBS_IN_WRAPPER contains non-defined jobs.  parameter is invalid"]]
+                if 'horizontal' in self.get_wrapper_type(wrapper_values):
+                    if not self.experiment_data["PLATFORMS"][platform_name].get('PROCESSORS_PER_NODE',"1"):
+                        self.wrong_config["WRAPPERS"] += [
+                            [wrapper_name, "PROCESSORS_PER_NODE no exist in the horizontal-wrapper platform"]]
+                    if not self.experiment_data["PLATFORMS"][platform_name].get('MAX_PROCESSORS',""):
                         self.wrong_config["WRAPPERS"] += [[wrapper_name,
-                                                          "JOBS_IN_WRAPPER contains non-defined jobs.  parameter is invalid"]]
-                    if 'horizontal' in self.get_wrapper_type(wrapper_values):
-                        if not self.experiment_data["PLATFORMS"][platform_name].get('PROCESSORS_PER_NODE',"1"):
-                            self.wrong_config["WRAPPERS"] += [
-                                [wrapper_name, "PROCESSORS_PER_NODE no exist in the horizontal-wrapper platform"]]
-                        if not self.experiment_data["PLATFORMS"][platform_name].get('MAX_PROCESSORS',""):
-                            self.wrong_config["WRAPPERS"] += [[wrapper_name,
-                                                              "MAX_PROCESSORS no exist in the horizontal-wrapper platform"]]
-                    if 'vertical' in self.get_wrapper_type(wrapper_values):
-                        if not self.experiment_data["PLATFORMS"][platform_name].get('MAX_WALLCLOCK',""):
-                            self.wrong_config["WRAPPERS"] += [[wrapper_name,
-                                                              "MAX_WALLCLOCK no exist in the vertical-wrapper platform"]]
-                if "WRAPPERS" not in self.wrong_config:
+                                                          "MAX_PROCESSORS no exist in the horizontal-wrapper platform"]]
+                if 'vertical' in self.get_wrapper_type(wrapper_values):
+                    if not self.experiment_data["PLATFORMS"][platform_name].get('MAX_WALLCLOCK',""):
+                        self.wrong_config["WRAPPERS"] += [[wrapper_name,
+                                                          "MAX_WALLCLOCK no exist in the vertical-wrapper platform"]]
+            if "WRAPPERS" not in self.wrong_config:
+                if not no_log:
                     Log.result('wrappers OK')
-                    return True
+                return True
     def file_modified(self,file,prev_mod_time):
         '''
         Function to check if a file has been modified.
@@ -1264,7 +1392,8 @@ class AutosubmitConfig(object):
         :return: a dictionary containing tuples [parameter_name, parameter_value]
         :rtype: dict
         """
-        return self.deep_parameters_export(self.experiment_data)
+        self.parameters = self.deep_parameters_export(self.experiment_data)
+        return self.parameters
 
     def load_platform_parameters(self):
         """
@@ -2065,16 +2194,16 @@ class AutosubmitConfig(object):
          """
         return wrapper.get('METHOD', self.experiment_data["WRAPPERS"].get("METHOD",'ASThread'))
 
-    def get_wrapper_check_time(self, wrapper=None):
+    def get_wrapper_check_time(self):
         """
          Returns time to check the status of jobs in the wrapper
 
          :return: wrapper check time
          :rtype: int
          """
-        if wrapper is None:
-            return 10
-        return int(wrapper.get('CHECK_TIME_WRAPPER', self.experiment_data["WRAPPERS"].get("CHECK_TIME_WRAPPER",self.get_safetysleeptime())))
+        wrapper = self.experiment_data.get("WRAPPERS", {})
+
+        return wrapper.get("CHECK_TIME_WRAPPER",self.get_safetysleeptime())
 
     def get_wrapper_machinefiles(self, wrapper={}):
         """
